@@ -296,7 +296,8 @@ def main(*args, **keywords):
     try:
         if args.pidfile:
             try:
-                open(args.pidfile, "w").write(str(os.getpid()))
+                with open(args.pidfile, "w") as f:
+                    f.write(str(os.getpid()))
             except OSError:
                 sys.stderr.write(_("failed to create PID file %s")
                                  % args.pidfile + "\n")
