@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2011-2015 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2011-2016 Mir Calculate. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -25,7 +25,9 @@ _ = lambda x: x
 setLocalTranslate('cl_core3', sys.modules[__name__])
 
 from calculate.lib.utils.files import listDirectory
-from calculate.lib.utils.portage import isPkgInstalled, getPkgSlot
+from calculate.lib.utils.portage import (isPkgInstalled, getPkgSlot,
+    getInstalledAtom)
+from calculate.lib.utils.text import formatListOr
 from calculate.lib.utils.common import getTupleVersion
 from calculate.lib.variables.user import VariableUrLogin
 from itertools import *
@@ -65,6 +67,52 @@ class VariableClCorePkgName(Variable):
                                          fullPath=True)))))
 
 
+class VariableClCorePkgVersionOpt(Variable):
+    """
+    Package version
+    """
+    type = "choiceedit"
+    opt = ["--pkg-version"]
+    metavalue = "PVR"
+    untrusted = True
+
+    def init(self):
+        self.label = _("Package version")
+        self.help = _("package version number with the revision")
+
+    def get(self):
+        return "all"
+
+    def choice(self):
+        pkg = getInstalledAtom("%s/%s" % (self.Get('cl_core_pkg_category'),
+                                        self.Get('cl_core_pkg_name')))
+        pkg = list(sorted(pkg))
+        if pkg:
+            return ["all"] + [x['PVR'] for x in pkg]
+        return ["all"]
+
+    def check(self, value):
+        pkg_name = self.Get('cl_core_pkg_name')
+        if not value and pkg_name and pkg_name != "all":
+            raise VariableError(_("Please choose the version"))
+        if value != "all":
+            pkg_name = self.Get('cl_core_pkg_name')
+            if not value and pkg_name and pkg_name != "all":
+                raise VariableError(_("Please choose the version"))
+
+            if self.Get('cl_action') != "patch":
+                pkgs = getInstalledAtom("%s/%s" % (self.Get('cl_core_pkg_category'),
+                                                self.Get('cl_core_pkg_name')))
+                pkgs = list(sorted(pkgs))
+                if pkgs:
+                    versions = [x['PVR'] for x in pkgs]
+                else:
+                    versions = []
+                if value not in versions and pkg_name and pkg_name != "all":
+                    raise VariableError(_("Please choose the version") + _(": ") +
+                        formatListOr(versions))
+
+
 class VariableClCorePkgVersion(Variable):
     """
     Package version
@@ -79,6 +127,9 @@ class VariableClCorePkgVersion(Variable):
         self.help = _("package version number with the revision")
 
     def get(self):
+        vers = self.Get('cl_core_pkg_version_opt')
+        if vers != "all":
+            return vers
         pkg = isPkgInstalled("%s/%s" % (self.Get('cl_core_pkg_category'),
                                         self.Get('cl_core_pkg_name')))
         if pkg:
@@ -94,7 +145,7 @@ class VariableClCorePkgVersion(Variable):
 
 class VariableClCorePkgSlot(Variable):
     """
-    Package version
+    Package Slot
     """
     type = "choiceedit"
     opt = ["--pkg-slot"]
@@ -106,6 +157,9 @@ class VariableClCorePkgSlot(Variable):
         self.help = _("package slot")
 
     def get(self):
+        slot = self.Get('cl_core_pkg_slot_opt')
+        if slot != "all":
+            return slot
         pkg = getPkgSlot("%s/%s" % (self.Get('cl_core_pkg_category'),
                                     self.Get('cl_core_pkg_name')))
         if pkg:
@@ -117,6 +171,66 @@ class VariableClCorePkgSlot(Variable):
         if not value and pkg_name and pkg_name != "all":
             raise VariableError(_("Please choose the slot"))
 
+class VariableClCorePkgSlotOpt(Variable):
+    """
+    Selected Package Slot
+    """
+    type = "choiceedit"
+    opt = ["--pkg-slot"]
+    metavalue = "SLOT"
+    untrusted = True
+
+    def init(self):
+        self.label = _("Package slot")
+        self.help = _("package slot")
+
+    def get(self):
+        veropt = self.Get('cl_core_pkg_version_opt')
+        if veropt != 'all':
+            pkgs = getInstalledAtom(
+                "=%s/%s-%s" % (self.Get('cl_core_pkg_category'),
+                self.Get('cl_core_pkg_name'), veropt))
+            pkgs = list(pkgs)
+            if pkgs:
+                return pkgs[0]["SLOTONLY"]
+        return "all"
+
+    def choice(self):
+        veropt = self.Get('cl_core_pkg_version_opt')
+        if veropt != "all":
+            pkg = getInstalledAtom("=%s/%s-%s" % (self.Get('cl_core_pkg_category'),
+                                            self.Get('cl_core_pkg_name'),
+                                            veropt))
+        else:
+            pkg = getInstalledAtom("%s/%s" % (self.Get('cl_core_pkg_category'),
+                                            self.Get('cl_core_pkg_name')))
+        pkg = list(sorted(pkg))
+        if pkg:
+            return ["all"] + [x['SLOTONLY'] for x in pkg]
+        return ["all"]
+
+
+    def check(self, value):
+        if value != "all":
+            pkg_name = self.Get('cl_core_pkg_name')
+            if not value and pkg_name and pkg_name != "all":
+                raise VariableError(_("Please choose the slot"))
+            veropt = self.Get('cl_core_pkg_version_opt')
+            if veropt == 'all':
+                pkgs = getInstalledAtom(
+                    "=%s/%s" % (self.Get('cl_core_pkg_category'),
+                                   self.Get('cl_core_pkg_name')))
+                slots = [x["SLOTONLY"] for x in sorted(pkgs)]
+            else:
+                pkgs = getInstalledAtom(
+                    "=%s/%s-%s" % (self.Get('cl_core_pkg_category'),
+                                   self.Get('cl_core_pkg_name'), veropt))
+                slots = [x["SLOTONLY"] for x in sorted(pkgs)]
+            if self.Get('cl_action') != "patch" and \
+                value not in slots and pkg_name and pkg_name != "all":
+                raise VariableError(_("Please choose the slot") + _(": ") +
+                    formatListOr(slots))
+
 
 class VariableClCorePkgCategory(Variable):
     """
