@@ -84,8 +84,8 @@ class Wsdl(WsdlBase):
                 lambda group: group(_("Configure a package"),
                                     normal=('cl_core_pkg_name',),
                                     expert=('cl_core_pkg_category',
-                                            'cl_core_pkg_version',
-                                            'cl_core_pkg_slot',
+                                            'cl_core_pkg_version_opt',
+                                            'cl_core_pkg_slot_opt',
                                             'cl_core_pkg_path',
                                             'cl_core_arch_machine',
                                             'cl_templates_locate',
